/*
 * @copyright        © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module            de_epages.presentation.inc.togglearea
 */

/**
 * Adds functionality to a toggle bar to switch between different tab areas
 *
 * ### Examples
 * toggles between two html divs.
 *
 * JavaScript:
 *
 * require([
 *   'jquery',
 *   'de_epages/presentation/inc/togglearea',
 *
 *   '$ready!'
 * ], function ($, toggleArea) {
 *   var defaultIndex = 0,
 *       tabOptions = [{
 *         buttonId: 'button1Id',
 *         frameId: 'tab1id'
 *       }, {
 *         buttonId: 'button2Id',
 *         frameId: 'tab2id'
 *       },
 *       ...
 *       ],
 *       myToggleArea = toggleArea({
 *         tabs: tabOptions,
 *         defaultIndex: defaultIndex
 *       });
 * });
 *
 * HTML:
 *
 *    <div class="ContentToggleArea">
 *        <a id="tab1Button" href="#" >tab1name</a>
 *        <a id="tab2Button" href="#" >tab2name</a>
 *    </div>
 *    <div id="tab1">
 *        *content*
 *    </div>
 *    <div id="tab2">
 *        *content*
 *    </div>
 *
 *
 *
 * @since 7.3.0
 */

 /*jslint browser: true, regexp: true*/
 /*global define*/

 define('de_epages/presentation/inc/togglearea', [
     'jquery'
 ], function ($) {
     'use strict';

    return function (options) {
        var tabs,
            onAfterToggle,
            toggleArea = function (targetIndex) {
                // iterate over tab Objects
                tabs.forEach(function(tab, index) {
                    // if active tab show tab and mark button as selected, hide otherwise
                    if (targetIndex === index) {
                        tab.frame.classList.remove('HideElement');
                        tab.button.classList.add('ShowArea');

                    } else {
                        tab.frame.classList.add('HideElement');
                        tab.button.classList.remove('ShowArea');
                    }
                });

                // execute after toggle callback
                if (onAfterToggle) {
                    onAfterToggle(targetIndex, tabs);
                }
            },
            init = function(options) {
                tabs = options.tabs;

                if (options.onAfterToggle && typeof options.onAfterToggle === "function") {
                    onAfterToggle = options.onAfterToggle;
                }

                // iterate over tab objects
                tabs.forEach(function(tab, index) {
                    // get dom-nodes from ID
                    tab.button = document.getElementById(tab.buttonId);
                    tab.frame = document.getElementById(tab.frameId);

                    // register click event handler
                    tab.button.addEventListener('click', function(e) {
                        toggleArea(index);
                        e.preventDefault();
                    });
                });

                // show default tab
                toggleArea(options.defaultIndex);
            };

        init(options);

        return {
            showAll: function() {
                // iterate over tab Objects and show all tabs
                tabs.forEach(function(tab, index) {
                    tab.frame.classList.remove('HideElement');
                });

                tabs[0].button.classList.add('ShowArea');
            }
        };
    };
});
